#!/bin/bash
#
# ----------------------------------------------------------
# Copyright (c) 2008-2012 Host Europe GmbH
# ----------------------------------------------------------
# Beschreibung:
# Cronjob zum Auslesen des RAID Status von perc[3|4|5|6] und
# 3ware Controllern. Der Output dieses Skripts wird von
# einem Nagios Plugin ausgelesen.
# ----------------------------------------------------------
# VERSION=2012-04-25
# Release History:
# 2012-04-25 VERSION hinzugefügt
# 2012-03-13 4/5/6/700/800: FWtermlog permanent auf Batterie setzen
# 2012-02-23 PERC4: ausgelagert
#            5/6/700/800: deutlich besser auslesen
#                         prefail auswerten
# 2011-01-13 RAID-Level ermitteln, Write-Cache-Status prüfen,
#            Rebuild-Status bei 'PARTIALLY DEGRADED' auslesen
# 2010-11-20 PERC4 aufwärts: Produktname auslesen
# 2010-11-12 PERC5 aufwärts: Controller-Uhrzeit setzen
# 2010-11-01 Batterie-Status bei PERC 5 aufwärts prüfen
# 2010-06-30 virtuelle Maschinen beruhigen
# 2010-06-01 PERC3 wird jetzt tatsächlich überprüft
# 2010-02-24 mptsas (SAS1068E) hinzugefügt
# 2010-02-17 Komplettes Redesign
# 2010-02-10 Großes Aufräumen
# 2009-03-27 Support fuer genaue Angabe des zu ueberpruefenden
#            Controllers, Adapters, Logicaldrives (perc[4|5|6])
#            und Angabe der Hotspares (perc[5|6])
# 2009-02-19 Support fuer PERC6 (=PERC5) hinzugefuegt
# 2009-02-16 Verbose Switch -v bei Programmaufruf eingefuehrt
#            Default Verhalten von OK auf CRITICAL geaendert
# 2008-11-04 Support fuer aacraid/PERC3 hinzugefuegt
# 2008-09-03 Bugfix fuer korrekte Verwendung der Architektur
#            Behandlung fuer mehrere Volumes am selben SCSI
#            Adapter fuer PERC 4/5
# 2008-09-01 Bugfix falls kein SCSI Adapter vorhanden ist
# 2008-04-29 Erstes Release
# ----------------------------------------------------------
# Authors:
# Michael Bakker <mba@hosteurope.de>
# Hendrik Cornelißen <hc@hosteurope.de>
# ----------------------------------------------------------

export PATH="/usr/lib/he-nagios/plugins/raid/:$PATH"

logdir="/var/cache/he-nagios-plugins-local";
NAGIOS_USER="he-nagios"
NAGIOS_GROUP="he-nagios"
touch /var/cache/he-nagios-plugins-local

# begin import from cron-hwreport.sh #
is_xen (){
        if test -d /sys/bus/xen/devices ; then
                true;
        else
                false;
        fi
}

is_xen_domU (){
        if is_xen; then
                if test -L /sys/bus/xen/devices/vif-0 ; then
                        true;
                else
                        false;
                fi
        else
                false;
        fi
}

is_xen_dom0 (){
        if is_xen; then
                if is_xen_domU ; then
                        false;
                else
                        true;
                fi
        else
                false;
        fi
}
# end import from cron-hwreport.sh #

usage () {
	printf 'Usage:  %s [-h|--help]\n' "${0}";
}

log_data () {
        hba="${1}";
        raid_state="${2}";
        logical_drive="${3}";
        hotspares="${4}";
        percent="${5}";
        absolute_charge="${6}";
        relative_charge="${7}";
        write_policy="${8}";
        raid_level="${9}";
        reason="${10}";
        adapter_product="${11}";
        predictive_failure_data="${12}";
        rebuild_data="${13}";
        
        raid_state="`printf '%s' \"${raid_state}\" | tr '[a-z]' '[A-Z]' | tr -cd '[A-Z] '`";
        write_policy="`printf '%s' \"${write_policy}\" | tr '[a-z]' '[A-Z]' | tr -cd '[A-Z]'`";
        
        
        [ x"${logical_drive}" = x"" ] && logical_drive="0";
        logfile="${logdir}/hba${hba}-logical_drive${logical_drive}";
        touch "${logfile}";
        chmod 0444 "${logfile}";
        chown $NAGIOS_USER "${logfile}";
        printf 'hba=%d\nraid_state="%s"\nlogical_drive=%d\nhotspares=%d\npercent=%s\nabsolute_charge=%d\nrelative_charge=%d\nwrite_policy=%s\nraid_level=%s\nreason="%s"\nadapter_product="%s"\npredictive_failure_data="%s"\nrebuild_data="%s"\n' \
                "${hba}" "${raid_state}" "${logical_drive}" "${hotspares}" "${percent}" "${absolute_charge}" "${relative_charge}" "${write_policy}" "${raid_level}" "${reason}" "${adapter_product}" \
                "${predictive_failure_data}" "${rebuild_data}" \
                > "${logfile}";
}               

check_binary () {
        hba="${1}";
        binary="${2}";
        if [[ ! -x $binary ]];then
            binary=$(which $binary)
        fi
        mode="${3}";
        [ ! -x "$binary" ] \
                && log_data "${hba}" '' '' '' '' '' '' '' '' '' "${binary} not found or not executable [${mode}]" \
                && return 1;
        return 0;
}

check_perc3 () {
	check_binary "${hba}" "afacli" || return;

	major="`sed -n -e 's/^\([0-9]*\) aac$/\1/p' /proc/devices`";
	if [ x"${major}" != x"" ];
	then
		rm -f /dev/afa0;
		mknod /dev/afa0 c "${major}" 0;
	fi;

	tempfile="`mktemp`";
	tempdir="`dirname \"${tempfile}\"`";
	tempname="`basename \"${tempfile}\"`";
	cd "${tempdir}";

	printf 'logfile start %s\nopen afa0\ncontainer list\nclose\nlogfile end\nexit\n' "${tempname}" \
		| env TERM=dumb "${binary}" > /dev/null 2>&1;
	awk '/^ [0-9]+/ { print $4 }' "${tempname}" | while read raid_state;
	do
		raid_state="`printf '%s' \"${raid_state}\" | tr '[a-z]' '[A-Z]'`";
		[ x"${raid_state}" = x"VALID" ] && raid_state="OPTIMAL";
		log_data "${hba}" "${raid_state}" '' '' '' '-1' '-1';
	done;
	rm -f "${tempname}";
}

check_perc456 () {
	hba="${1}";
	if [ x"${mode}" = x"perc4" ];
	then
		major="`sed -n -e 's/^\([0-9]*\) megadev$/\1/p' /proc/devices`";
		if [ x"${major}" != x"" ];
		then
			rm -f /dev/megadev0;
			mknod /dev/megadev0 c "${major}" 0;
		fi;
	fi;

	if [ x"${mode}" = x"perc4" ];
	then
		binary="megacli32-perc4";
	else
		architecture=$(uname -m)
		if [[ -z $architecture  ]]; then
			log_data "${hba}" '' '' '' '' '' '' '' '' "Unable to determine architecture [${mode}]"
			return
		elif [[ $architecture == "x86_64" || $architecture == ia64 ]]; then
			architecture=64
		else
			architecture=32
		fi
		binary="megacli${architecture}-perc5";
	fi;
	check_binary "${hba}" "${binary}" || return;

	absolute_charge=-1;
	relative_charge=-1;

	if [ x"${mode}" = x"perc4" ];
	then
		number_of_logical_drives="`${binary} -LogPhysInfo -a${hba} | sed -n -e 's/.*Total Logical Drive.* //p'`";

		adapter_product="`${binary} -AllAdpInfo -a${hba} | sed -n -e 's/.*PERC/PERC/p'`";
	else
		number_of_logical_drives="`${binary} -LDGetNum -a${hba} -NoLog | sed -n -e 's/.*Number.*[aA]dapter.*: //p'`";
		
		adapter_product="`${binary} -AdpAllInfo -a${hba} -NoLog | sed -n -e 's/.*Product Name [ ]*: //p'`";
		current_date="`date '+%m%d%Y%H%M'`";
		controller_date="`${binary} -AdpGetTime -a${hba} -NoLog | sed -n -e 's/.*Date: //p' -e 's/.*Time: \(..:..\):.*/\1/p' | tr -cd '[0-9]'`";
		[ x"${current_date}" != x"${controller_date}" ] && ${binary} -AdpSetTime "`date '+%Y%m%d %H:%M:%S'`" -a${hba} -Silent -NoLog >/dev/null;

		charge_data="`${binary} -AdpBbuCmd -GetBbuCapacityInfo -a${hba} -NoLog`";
		absolute_charge="`printf '%s' \"${charge_data}\" | sed -n -e 's/.*Absolute State of charge: \(.*\) %$/\1/p'`";
		relative_charge="`printf '%s' \"${charge_data}\" | sed -n -e 's/.*Relative State of Charge: \(.*\) %$/\1/p'`";
	fi;

	[ x"${number_of_logical_drives}" = x"0" ] && return;
	[ x"${number_of_logical_drives}" = x"" ] \
		&& log_data "${hba}" '' '' '' '' '' '' '' '' "no output for logical drives [${mode}]" \
		&& return;
	number_of_logical_drives="`expr \"${number_of_logical_drives}\" '-' 1`";

	for logical_drive in `seq 0 "${number_of_logical_drives}"`;
	do
		hotspares='';

		if [ x"${mode}" = x"perc4" ];
		then
			raid_data="`${binary} -ldInfo -L${logical_drive} -a${hba} 2>&1`";
			raid_state="`printf '%s' \"${raid_data}\" | sed -n -e 's/.* Status: //p'`";
			raid_level="`printf '%s' \"${raid_data}\" | sed -n -e 's/.* RaidLevel: \([0-9]*\) .*/\1/p'`";
			write_policy="`printf '%s' \"${raid_data}\" | sed -n -e 's/.* WrPolicy: //p'`";
		else
			raid_data="`${binary} -LDInfo -L${logical_drive} -a${hba} -NoLog 2>&1`";
			raid_state="`printf '%s' \"${raid_data}\" | sed -n -e 's/^State: //p'`";
			raid_level="`printf '%s' \"${raid_data}\" | sed -n -e 's/^RAID Level: //p' | tr -cd '[0-9]'`";
			write_policy="`printf '%s' \"${raid_data}\" | sed -n -e 's/^Current Cache Policy: \([a-zA-Z]*\),.*/\1/p'`";
			hotspares="`printf '%s' \"${raid_data}\" | sed -n -e 's/^Number of Dedicated Hot Spares: //p'`";
		fi;

		raid_state="`printf '%s' \"${raid_state}\" | tr '[a-z]' '[A-Z]' | tr -cd '[A-Z] '`";
		raid_level="`printf '%s' \"${raid_level}\" | sed -e 's/03//' -e 's/30/0/' -e 's/100/1/'`";
		write_policy="`printf '%s' \"${write_policy}\" | tr '[a-z]' '[A-Z]' | tr -cd '[A-Z]'`";

		percent=0;
		if [ x"${raid_state}" = x"DEGRADED" ] || [ x"${raid_state}" = x"PARTIALLY DEGRADED" ] && [ x"${mode}" = x"perc[5|6]" ];
		then
			disks="`${binary} -PDList -a${hba} -NoLog | tr '\n' '|' | sed -e 's/||/\n/g' | awk -F'[|:]' '/Rebuild/ { print $2 \":\" $4 }' \
				| tr -d '       '`";
			number_of_disks=0;
			for disk in ${disks};
			do
				disk_percent="`${binary} -PDRbld -ShowProg -PhysDrv [${disk}] -a${hba} -NoLog | sed -n -e 's/.*Completed \([0-9]*\)% in .*/\1/p'`";
				number_of_disks="`expr \"${number_of_disks}\" '+' 1`";
				percent="`expr \"${percent}\" '+' \"${disk_percent}\"`";
			done;
			[ x"${number_of_disks}" != x"0" ] && percent="`expr \"${percent}\" '/' \"${number_of_disks}\"`";
		fi;
		log_data "${hba}" "${raid_state}" "${logical_drive}" "${hotspares}" "${percent}" "${absolute_charge}" "${relative_charge}" "${write_policy}" "${raid_level}" '' "${adapter_product}";
	done;
}

# check_3ware () { #
#	hba="${1}"; #
#	binary='tw_cli'; #
#	check_binary "${hba}" "${binary}" || return; #
# #
#	[ x"`${binary} help | sed -n -e 's/.*3ware CLI.*version \(.*\))$/\1/p'`" != x"2.00.03.017" ] \ #
#		&& log_data "${hba}" '' '' '' '' '' '' '' '' "${binary} is not the required version [${mode}]" \ #
#		&& return; #
# #
#	state="`${binary} info c${hba} unitstatus 2>&1`"; #
#	raid_state="`printf '%s' \"${state}\" | awk '/^u/ { print $3 }' | tr '[a-z]' '[A-Z]'`"; #
#	[ x"${raid_state}" = x"OK" ] && raid_state="OPTIMAL"; #
#	percent="`printf '%s' \"${state}\" | awk '/^u/ { print $4 }'`"; #
#	log_data "${hba}" "${raid_state}" '' '' "${percent}" '-1' '-1'; #
#} #

check_mptsas () {
	hba="${1}";
	binary='/usr/sbin/mpt-status';
	check_binary "${hba}" "${binary}" || return;
	state="`${binary} | head -n 1`";
	raid_state="`printf '%s' \"${state}\" | sed -n -e 's/.* state \([A-Z]*\), .*/\1/p'`";
	raid_level="`printf '%s' \"${state}\" | sed -n -e 's/.*type IM.*/1/p' -e 's/.*type IS.*/0/p'`";
	[ x"${raid_state}" = x"DEGRADED" ] && flags="`printf '%s' \"${state}\" | sed -n -e 's/.* flags .* RESYNC_IN_PROGRESS/REBUILDING/p'`";
	[ x"${flags}" = x"REBUILDING" ] && raid_state="${flags}";
	log_data "${hba}" "${raid_state}" '' '' '' '-1' '-1' '' "${raid_level}" '' 'SAS 1068E?';
}

[ x"${1}" = x"-h" ] || [ x"${1}" = x"--help" ] && usage && exit 0;

[ x"${UID}" != x"0" ] && printf 'Eh, you are not root!\n' 1>&2 && exit 1;

cd /tmp;
mkdir -p "${logdir}";
chmod 0755 "${logdir}";
chown $NAGIOS_USER:$NAGIOS_GROUP  "${logdir}";

# virtuelle Gastmaschinen nicht weiter belästigen #
is_xen_domU && exit 0;

[ ! -d /sys ] && printf 'SYSFS is not available\n' 1>&2 && exit 1;
[ ! -d /sys/class/scsi_host ] && exit 1;

hbas_to_check="`find /sys/class/scsi_host -maxdepth 1 -mindepth 1 -name 'host*' \
	| sed -e 's+/sys/class/scsi_host/host++' | tr '\n' ' '`";
[ x"${hbas_to_check}" = x"" ] && exit 0;


for hba in ${hbas_to_check};
do
	proc_name="`cat /sys/class/scsi_host/host${hba}/proc_name 2>/dev/null`";
	stats="`grep '3w-xxxx' /sys/class/scsi_host/host${hba}/stats 2>/dev/null`";

	if [ x"${proc_name}" = x"aacraid" ];
	then
		mode="perc3";
		check_perc3 "${hba}";
	elif [ x"${proc_name}" = x"megaraid" ];
	then
		mode="perc4";
		check_perc456 "${hba}";
	elif [ x"${proc_name}" = x"megaraid_sas" ];
	then
		mode="perc[5|6]";
		check_perc456 "${hba}";
	elif [ x"${proc_name}" = x"mptsas" ];
	then
		mode="mptsas";
		check_mptsas "${hba}";
	fi;
done;
rm -f MegaSAS.log megarc.log;
#	elif [ x"${stats}" != x"" ]; #
#	then #
#		mode="3ware"; #
#		check_3ware "${hba}"; #
